/*
 * edplyr.c
 * Copyright (C) Damian Walker 1998
 *
 * Astroserv - Player Database Edit Module
 *
 * $Id: edplyr.c,v 1.3 2001/02/15 16:38:23 damian Exp $
 *
 * $Log: edplyr.c,v $
 * Revision 1.3  2001/02/15 16:38:23  damian
 * Cleaned up logging and screen output
 *
 * Revision 1.2  2000/07/27 09:31:24  damian
 * Cosmetic changes to heading comments
 *
 * Revision 1.1.1.1  2000/07/27 09:18:08  damian
 * Initial release of Astroserv
 *
 * Revision 1.1  2000/05/31 10:17:51  damian
 * Initial revision
 *
 */


/* included headers ***************************************************/


#include <stdio.h>
#include <string.h>
#include <astrowar.h>
#include <astrowar/gamlst.h>
#include "astrosrv.h"
#include "config.h"
#include "player.h"
#include "scrlog.h"


/* enumerated types ***************************************************/


/* edplayer services */
typedef enum {
  S_NONE,   /* no service requested */
  S_ADD,    /* add a new player record */
  S_EDIT,   /* edit a player record */
  S_DELETE, /* delete a player record */
  S_LIST,   /* list all or selected player records */
  S_ERROR   /* error in command line */
} service;


/* level 1 routines ***************************************************/


/* edplyr_open() - open player editor */
result edplyr_open(char *gamepath, char *path, char *gamename)
{
  result
    r;                /* returned to calling process */
  char
    logfilename[128]; /* ename and path of log file */
  int
    line;             /* line on which config read failed */

  /* attempt to read configuration */
  if(( r = config_open(path) ))
    {
      scrlog_badreturncode(r, "config_open");
      return r;
    }
  switch( r = config_read(path, &line) )
    {
    case R_FILE:
      scrlog_cannotreadconfig(path);
      return r;
    case R_BADCMD:
      scrlog_badcommandinconfig(path, line);
      return r;
    case R_OK:
      break;
    default:
      scrlog_badreturncode(r, "config_read");
      return r;
    }

  /* attempt to open log file */
  config_getlogfile(logfilename);
  if(( r = scrlog_openlog(logfilename, config_getloglevel ()) ))
    {
      scrlog_cannotopenlog(logfilename);
      goto closeconfig;
    }
  scrlog_edplyr();

  /* open game list and find game */
  switch( r = gamlst_open(path, &line) )
    {
    case R_OK:
      scrlog_gamelistread(path);
      break;
    case R_FILE:
      scrlog_cannotopengamelist(path);
      goto closeconfig;
    case R_EOF:
      scrlog_badfileingamelist(path, line);
      goto closeconfig;
    case R_BADCMD:
      scrlog_badlineingamelist(path, line);
      goto closeconfig;
    default:
      scrlog_badreturncode(r, "gamlst_open");
      goto closeconfig;
    }
  if(( r = gamlst_find(gamename) ))
    {
      scrlog_gamenotfound(gamename);
      goto closegamelist;
    }
  gamlst_getpath(gamepath);
  gamlst_close();

  /* open player file */
  switch( r = player_open(gamepath) )
    {
    case R_FILE:
      if(!( r = player_create(gamepath) ))
	{
          scrlog_filecreated(gamepath, "splayer.data");
          if(!( r = player_open(gamepath) ))
	    {
              scrlog_fileopened(gamepath, "splayer.data");
	      break;
	    }
	}
      scrlog_cannotopenfile(gamepath, "splayer.data");
      goto closegamelist;
    case R_OK:
      scrlog_fileopened(gamepath, "splayer.data");
      break;
    case R_HEADER:
      scrlog_badheaderinfile(gamepath, "splayer.data");
      goto closegamelist;
    default:
      scrlog_badreturncode(r, "player_open");
      goto closegamelist;
    }

  /* all OK, return */
  return R_OK;

  /* deal with errors */
closegamelist:
  gamlst_close();
closeconfig:
  config_close();
  return r;
}

/* edplyr_interpret() - interpret command line arguments */
service edplyr_interpret(char *gamename, char *ename, long *unum, char **argp)
{
  int
    argc,    /* argument count */
    argn;    /* number of arguments */
  service
    s;       /* service to return */
  long
    num;     /* number read from command line */
  enum {     /* state variable */
    T_NONE,  /* not expecting anything in particular */
    T_GAME,  /* expecting a game ename */
    T_ENAME, /* expecting an empire ename */
    T_UNUM   /* expecting a user number */
  } t;

  /* initialise variables */
  s = S_NONE;
  *unum = 0;
  strcpy(ename, "");
  strcpy(gamename, "");
  t = T_NONE;

  /* find last argument */
  argn = 10;
  for(argc = 9; argc >= 0; --argc) if( !argp[argc] ) argn = argc;

  /* interpret arguments */
  for(argc = 0; argc < argn && s != S_ERROR; ++argc)
    {
      if( !strcasecmp( argp[argc], "--add" ) && s == S_NONE )
	s = S_ADD;
      else if( !strcasecmp( argp[argc], "--edit" ) && s == S_NONE )
	s = S_EDIT;
      else if( !strcasecmp( argp[argc], "--delete" ) && s == S_NONE )
	s = S_DELETE;
      else if( !strcasecmp( argp[argc], "--list" ) && s == S_NONE )
	s = S_LIST;
      else if( sscanf( argp[argc], "%ld", &num ) == 1 && !*unum )
	*unum = num;
      else if( !strcasecmp( argp[argc], "--empire" ) && t == T_NONE )
        t = T_ENAME;
      else if( t == T_ENAME )
        { strncpy( ename, argp[argc], 16 ); t = T_NONE; }
      else if( !*gamename )
        { strncpy( gamename, argp[argc], 16 ); t = T_NONE; }
      else
	s = S_ERROR;
    }
  if(t) s = S_ERROR;

  return s;
}

/* edplyr_add() - add a player record to the player database */
void edplyr_add(char *ename, long unum)
{
  player
    *p = player_new(); /* new player record */

  /* log action */
  scrlog_playeradd(ename);

  /* validate player details */
  if(!*ename || !unum)
    {
      scrlog_badplayerdetails(ename);
      player_old(p);
      return;
    }

  /* create player details */
  player_clear(p);
  player_setempire(p, ename);
  player_setnumber(p, unum);

  /* save and clean up */
  player_write(p);
  player_old(p);
}

/* edplyr_edit() - edit a player record in the player database */
void edplyr_edit(char *ename, long unum)
{
  player
    *p = player_new(); /* player record to edit */

  /* log message */
  scrlog_playeredit(ename);

  /* find player record in file */
  if( player_find(p, ename) )
    {
      player_old(p);
      scrlog_badplayerdetails(ename);
      return;
    }

  /* change settings which were specified */
  if(*ename)
    player_setempire(p, ename);
  if(unum)
    player_setnumber(p, unum);

  /* save record and clean up */
  player_write(p);
  player_old(p);
}

/* edplyr_delete() - delete a player from the player database */
void edplyr_delete(char *ename, long unum)
{
  player *p = player_new(); /* player to delete */
  int deleteyes;        /* !0 if the player should be deleted */

  /* log message */
  scrlog_playerdelete(ename);

  /* find player record */
  if( player_find(p, ename) )
    {
      scrlog_badplayerdetails(ename);
      player_old(p);
      return;
    }

  /* verify player details (supplied as safety catch) */
  deleteyes = !0;
  if( unum != player_getnumber(p) ) deleteyes = 0;

  /* delete record and clean up */
  if(deleteyes)
    player_delete(p);
  else
    scrlog_badplayerdetails(ename);
  player_old(p);
}

/* edplyr_list() - list players in the player database */
void edplyr_list(char *ename, long unum)
{
  player *p = player_new(); /* player record */
  result r;                 /* result of file read */
  long uunum;               /* user number in player file */
  char uname[16];           /* player ename in player file */
  int listyes;              /* !0 = list this record */

  scrlog_plistheader();
  r = player_first(p, P_EMPIRE);
  while(!r)
    {
      /* get details */
      player_getempire(uname, p);
      uunum = player_getnumber(p);

      /* check against selection criteria */
      listyes = 1;
      if(unum && uunum != unum)
	listyes = 0;
      else if(*ename && strcasecmp(uname, ename) )
	listyes = 0;

      /* write record if OK & move on */
      if(listyes)
        scrlog_plistbody(uname, uunum);
      r = player_next(p, P_EMPIRE);
    }
  scrlog_plistfooter();

  /* clean up and return */
  player_old(p);
}

/* edplyr_close() - close files in use */
void edplyr_close(void)
{
  player_close();
  gamlst_close();
  config_close();
  scrlog_finished();
}


/* level 0 routines ***************************************************/


/* edplyr_main() - main player editor function */
result edplyr_main(char *path, char **argp)
{
  result r;        /* returned to calling process */
  long unum;       /* user number */
  char
    ename[16],     /* empire ename */
    gamepath[128], /* path of game to edit */
    gamename[16];  /* ename of game to edit players in */
  service s;       /* service requested */

  s = edplyr_interpret(gamename, ename, &unum, argp);
  if(( r = edplyr_open(gamepath, path, gamename) )) return r;
  switch(s)
    {
    case S_NONE: case S_ERROR:
      scrlog_error();
      return R_BADCMD;
    case S_ADD:
      edplyr_add(ename, unum);
      break;
    case S_EDIT:
      edplyr_edit(ename, unum);
      break;
    case S_DELETE:
      edplyr_delete(ename, unum);
      break;
    case S_LIST:
      edplyr_list(ename, unum);
      break;
    }
  edplyr_close();
  if(s == S_DELETE) player_pack(gamepath);
  return R_OK;
}
