/*
 * scrlog.c
 * Copyright (C) Damian Walker 2000
 *
 * Astroserv - Screen and Log Output Module
 *
 * $Id: scrlog.c,v 1.3 2001/02/15 16:38:25 damian Exp $
 *
 * $Log: scrlog.c,v $
 * Revision 1.3  2001/02/15 16:38:25  damian
 * Cleaned up logging and screen output
 *
 * Revision 1.2  2000/07/27 09:31:41  damian
 * Cosmetic changes to heading comments
 *
 * Revision 1.1.1.1  2000/07/27 09:18:12  damian
 * Initial release of Astroserv
 *
 * Revision 1.1  2000/05/31 10:17:51  damian
 * Initial revision
 *
 */


/* included headers ***************************************************/


#include <stdio.h>
#include "astrosrv.h"
#include "log.h"


/* global variables ***************************************************/


extern char pid[36];
static int logopen = 0;


/* main open/close routines *******************************************/


/* scrlog_open() - open screen and log */
result scrlog_open(void)
{
    setbuf(stdout, NULL);

    printf("\nCyningstan %s\n", pid);
    puts(  "Copyright (C) Damian Gareth Walker 2020");
    puts(  "URL: http://damian.cyningstan.org.uk/");
    puts(  "");

    return R_OK;
}

/* scrlog_openlog() - open the log file */
result scrlog_openlog(char *filename, short status)
{
    char fullpid[81];   /* "Cyningstan " + pid */

    sprintf(fullpid, "Cyningstan %s", pid);
    if( openlog(filename, fullpid) == LOGFILEERROR ) return R_FILE;
    setlogstatus(status);
    logopen = 1;

    return R_OK;
}

/* scrlog_close() - close the library */
void scrlog_close(void)
{
    if(logopen) closelog();
}

/* void scrlog_cannotopenlog() - cannot open the log file */
void scrlog_cannotopenlog(char *filename)
{
    printf("! Cannot open log file %s\n", filename);
}


/* environment-related messages ***************************************/


/* scrlog_error() - command line error */
void scrlog_error(void)
{
    puts("! Command line error: type \'astrosrv ?\' for help");
}

/* scrlog_running() - program already running */
void scrlog_running(void)
{
    puts("! Astroserv already running");
}

/* scrlog_help() - program help output */
void scrlog_help(void)
{
    puts("usage: astrosrv [<switches>] [params]");
    puts("");
    puts("switches =");
    puts("    --in        process inbound order messages");
    puts("    --out       generate outbound report messages");
    puts("    --user      edit user database");
    puts("    --player    edit player database");
    puts("    --help      request this help");
    puts("    --path      path to configs, semaphores etc.");
    puts("    --add       add user or player");
    puts("    --edit      edit user or player");
    puts("    --delete    delete user or player");
    puts("    --list      list users or players");
    puts("    --empire    empire name for player");
    puts("    --addr      address of user");
    puts("");
    puts("see manual for complete information");
}


/* general messages ***************************************************/


/* scrlog_turnin() - running turn input module */
void scrlog_turnin(void)
{
    char entry[128]; /* line for log and screen */

    sprintf(entry, "Running turn input module");
    printf("* %s\n", entry);
    writelog(LOGMAJOR, entry);
}

/* scrlog_trnout() - running turn output module */
void scrlog_trnout(void)
{
    char entry[128]; /* line for log and screen */

    sprintf(entry, "Running turn output module");
    printf("* %s\n", entry);
    writelog(LOGMAJOR, entry);
}

/* scrlog_eduser() - running database editor */
void scrlog_eduser(void)
{
    char entry[128]; /* line for log and screen */

    sprintf(entry, "Running user database editor");
    printf("* %s\n", entry);
    writelog(LOGMAJOR, entry);
}

/* scrlog_edplyr() - running database editor */
void scrlog_edplyr(void)
{
    char entry[128]; /* line for log and screen */

    sprintf(entry, "Running player database editor");
    printf("* %s\n", entry);
    writelog(LOGMAJOR, entry);
}

/* scrlog_outofmemory() - general out of memory error */
void scrlog_outofmemory(char *act)
{
    char entry[128]; /* line for log and screen */

    sprintf(entry, "Out of memory %s", act);
    printf("! %s\n", entry);
    writelog(LOGERROR, entry);
}

/* scrlog_badreturncode() - unexpected result code from function */
void scrlog_badreturncode(result r, char *function)
{
    char entry[128]; /* line for log and screen */

    sprintf(entry, "Bad return code %d from %s", r, function);
    printf("! %s\n", entry);
    writelog(LOGERROR, entry);
}

/* scrlog_finished() - finished */
void scrlog_finished(void)
{
    char entry[128]; /* line for log and screen */

    sprintf(entry, "Finished!");
    printf("- %s\n", entry);
    writelog(LOGTRIVIAL, entry);
}


/* config-related messages ********************************************/


/* scrlog_badcommandinconfig() - bad command in config file */
void scrlog_badcommandinconfig(char *path, int line)
{
    printf("! Bad command in line %d of %sastrosrv.data", line, path);
}

/* scrlog_cannotreadconfig() - file error opening config */
void scrlog_cannotreadconfig(char *path)
{
    printf("! Cannot read config file %sastrosrv.data\n", path);
}

/* scrlog_cannotwriteconfig() - file error creating config */
void scrlog_cannotwriteconfig(char *path)
{
    printf("! Cannot write config file %sastrosrv.data\n", path);
}

/* scrlog_configread() - config read successfully */
void scrlog_configread(char *path)
{
    char entry[128]; /* line for log entry */

    sprintf(entry, "Config file %sastrosrv.data read successfully",
        path);
    printf("- %s\n", entry);
    writelog(LOGTRIVIAL, entry);
}


/* detail-related errors **********************************************/


/* scrlog_cannotopendetail() - cannot read detail file */
void scrlog_cannotopendetail(char *path)
{
    char entry[128]; /* line for log and screen */

    sprintf(entry, "Cannot open detail file %sdetail.data", path);
    printf("! %s\n", entry);
    writelog(LOGERROR, entry);
}

/* scrlog_badcommandindetail() - bad command in detail file */
void scrlog_badcommandindetail(char *path)
{
    char entry[128]; /* line for log and screen */

    sprintf(entry, "Bad command in detail file %sdetail.data", path);
    printf("! %s\n", entry);
    writelog(LOGERROR, entry);
}

/* scrlog_detailread() - detail file read successfully */
void scrlog_detailread(char *path)
{
    char entry[128]; /* line for log and screen */

    sprintf(entry, "Detail file %sdetail.data read successfully", path);
    printf("- %s\n", entry);
    writelog(LOGTRIVIAL, entry);
}


/* game list file related messages ************************************/


/* scrlog_cannotopengamelist() - error opening game list */
void scrlog_cannotopengamelist(char *path)
{
    char entry[128]; /* line for log and screen */

    sprintf(entry, "Cannot open game list %sgamelist.data", path);
    printf("! %s\n", entry);
    writelog(LOGERROR, entry);
}

/* scrlog_badfileingamelist() - bad command in game list */
void scrlog_badfileingamelist(char *path, int line)
{
    char entry[128]; /* line for log and screen */

    sprintf(entry, "Bad detail file (line %d in %sgamelist.data)", line,
        path);
    printf("! %s\n", entry);
    writelog(LOGERROR, entry);
}

/* scrlog_badlineingamelist() - bad command in game list */
void scrlog_badlineingamelist(char *path, int line)
{
    char entry[128]; /* line for log and screen */

    sprintf(entry, "Bad line %d in game line %sgamelist.data", line,
        path);
    printf("! %s\n", entry);
    writelog(LOGERROR, entry);
}

/* scrlog_gamelistread() - game list read successfully */
void scrlog_gamelistread(char *path)
{
    char entry[128]; /* line for log and screen */

    sprintf(entry, "Game list %sgamelist.data read successfully", path);
    printf("- %s\n", entry);
    writelog(LOGTRIVIAL, entry);
}

/* scrlog_gamelistempty() - game list empty */
void scrlog_gamelistempty(char *path)
{
    char entry[128]; /* line for log and screen */

    sprintf(entry, "Game list %sgamelist.data is empty", path);
    printf("? %s\n", entry);
    writelog(LOGWARNING, entry);
}

/* scrlog_gamenotfound() - game not found */
void scrlog_gamenotfound(char *gamename)
{
    char entry[128]; /* line for log and screen */

    sprintf(entry, "Game \"%s\" not found", gamename);
    printf("- %s\n", entry);
    writelog(LOGTRIVIAL, entry);
}

/* scrlog_cannotwritegamelist() - cannot write game list */
void scrlog_cannotwritegamelist(char *path)
{
    char entry[128]; /* line for log and screen */

    sprintf(entry, "Cannot write game list %sgamelist.data", path);
    printf("? %s\n", entry);
    writelog(LOGWARNING, entry);
}


/* msglib/iomsg errors ************************************************/


/* scrlog_cannotopenfolder() - cannot open folder */
void scrlog_cannotopenfolder(char *filename)
{
    char entry[128]; /* line for log and screen */

    sprintf(entry, "Cannot open folder %s", filename);
    printf("! %s\n", entry);
    writelog(LOGERROR, entry);
}

/* scrlog_folderopened() - folder opened successfully */
void scrlog_folderopened(char *filename)
{
    char entry[128]; /* line for log and screen */

    sprintf(entry, "Folder %s opened successfully", filename);
    printf("- %s\n", entry);
    writelog(LOGTRIVIAL, entry);
}

/* scrlog_scanning() - scanning inbound messages */
void scrlog_scanning(void)
{
    printf("Scanning...\r");
}

/* scrlog_msgread() - message read */
void scrlog_msgread(char *addr)
{
    char entry[128]; /* line for log and screen */

    sprintf(entry, "Processing mail from %s", addr);
    printf("* %s\n", entry);
    writelog(LOGMAJOR, entry);
}

/* scrlog_processingorder() - processing order <whatever> */
void scrlog_processingorder(char *ordertype)
{
    char entry[128]; /* line for log and screen */

    sprintf(entry, "Processing \"%s\" order", ordertype);
    printf("- %s ...%15s\r", entry, "");
    writelog(LOGTRIVIAL, entry);
}

/* scrlog_badorder() - bad order in message */
void scrlog_badorder(int line)
{
    char entry[128]; /* line for log and screen */

    sprintf(entry, "Rejecting bad order in line %d of message", line);
    printf("+ %s\n", entry);
    writelog(LOGMINOR, entry);
}

/* scrlog_messagedone() - finished processing message */
void scrlog_messagedone(void)
{
    char entry[128]; /* line for log and screen */

    sprintf(entry, "Done processing this message");
    printf("- %s\n", entry);
    writelog(LOGTRIVIAL, entry);
}

/* scrlog_replyposted() - reply posted */
void scrlog_replyposted(char *addr)
{
    char entry[128]; /* line for log and screen */

    sprintf(entry, "Reply posted to %s", addr);
    printf("+ %s\n", entry);
    writelog(LOGMINOR, entry);
}

/* scrlog_reportposted() - report posted */
void scrlog_reportposted(char *addr)
{
    char entry[128]; /* line for log and screen */

    sprintf(entry, "Report posted to %s", addr);
    printf("* %s\n", entry);
    writelog(LOGMAJOR, entry);
}

/* scrlog_scancomplete() - netmail scan complete */
void scrlog_scancomplete(void)
{
    char entry[128]; /* line for log and screen */

    sprintf(entry, "Mail scan complete");
    printf("* %-72s\n", entry);
    writelog(LOGMINOR, entry);
}

/* scrlog_baduserdetails() - bad user details */
void scrlog_baduserdetails(long usernum)
{
    char entry[128]; /* line for log and screen */

    sprintf(entry, "Unmatched or missing details for user %ld", usernum);
    printf("? %s\n", entry);
    writelog(LOGWARNING, entry);
}

/* scrlog_badplayerdetails() - bad player details */
void scrlog_badplayerdetails(char *ename)
{
    char entry[128]; /* line for log and screen */

    sprintf(entry, "Unmatched or missing details for player %s", ename);
    printf("? %s\n", entry);
    writelog(LOGWARNING, entry);
}


/* general file-related messages **************************************/


/* scrlog_cannotopenfile() - cannot open file.* */
void scrlog_cannotopenfile(char *path, char *file)
{
    char entry[128]; /* line for log and screen */

    sprintf(entry, "Cannot open file %s%s", path, file);
    printf("! %s\n", entry);
    writelog(LOGERROR, entry);
}

/* scrlog_badheaderinfile() - bad header in file.* */
void scrlog_badheaderinfile(char *path, char *file)
{
    char entry[128]; /* line for log and screen */

    sprintf(entry, "Bad header in file %s%s", path, file);
    printf("! %s\n", entry);
    writelog(LOGERROR, entry);
}

/* scrlog_fileopened() - join request file opened */
void scrlog_fileopened(char *path, char *file)
{
    char entry[128]; /* line for log and screen */

    sprintf(entry, "File %s%s opened successfully", path, file);
    printf("- %s\n", entry);
    writelog(LOGTRIVIAL, entry);
}

/* scrlog_fileexists() - join request file exists */
void scrlog_fileexists(char *path, char *file)
{
    char entry[128]; /* line for log and screen */

    sprintf(entry, "file %s%s already exists", path, file);
    printf("! %s\n", entry);
    writelog(LOGERROR, entry);
}

/* scrlog_cannotcreatefile() - cannot create join request file */
void scrlog_cannotcreatefile(char *path, char *file)
{
    char entry[128]; /* line for log and screen */

    sprintf(entry, "Cannot create file %s%s", path, file);
    printf("! %s\n", entry);
    writelog(LOGERROR, entry);
}

/* scrlog_filecreated() - join request file created */
void scrlog_filecreated(char *path, char *file)
{
    char entry[128]; /* line for log and screen */

    sprintf(entry, "File %s%s created successfully", path, file);
    printf("+ %s\n", entry);
    writelog(LOGMINOR, entry);
}

/* scrlog_purgedfile() - file purged */
void scrlog_purgedfile(char *path, char *file)
{
    char entry[128]; /* line for log and screen */

    sprintf(entry, "File %s%s purged of deleted records", path, file);
    printf("+ %s\n", entry);
    writelog(LOGMAJOR, entry);
}


/* 'turnout' specific messages ****************************************/


/* scrlog_processingreport() - processing <whatever> report */
void scrlog_processingreport(char *name)
{
    char entry[128]; /* line for log and screen */

    sprintf(entry, "Processing \"%s\" report", name);
    printf("- %s... %15s\r", entry, "");
    writelog(LOGTRIVIAL, entry);
}

/* scrlog_donereports() - done processing reports */
void scrlog_donereports(void)
{
    char entry[128]; /* line for log and screen */

    sprintf(entry, "All reports have been processed");
    printf("- %s\n", entry);
    writelog(LOGTRIVIAL, entry);
}

/* scrlog_readingorderfile() - reading order file */
void scrlog_readingorderfile(void)
{
    printf("- Reading order file ...\r");
}


/* user database related messages *************************************/


/* scrlog_listheader() - display user list header */
void scrlog_listheader(void)
{
  char entry[128]; /* line for log and screen */

  /* "this is a list" log message */
  strcpy(entry, "Listing users");
  printf("+ %s\n", entry);
  writelog(LOGMINOR, entry);

  /* list header on the screen */
  puts("---usernum-email-----------------------"
       "---------------------------------------");
}

/* scrlog_listbody() - display an entry in the user list body */
void scrlog_listbody(long unum, char *addr)
{
  printf("%10ld %-60s\n", unum, addr);
}

/* scrlog_listfooter() - display the list footer */
void scrlog_listfooter(void)
{
  puts("---------------------------------------"
       "---------------------------------------");
}

/* scrlog_useradd() - adding user ... */
void scrlog_useradd(long usernum)
{
    char entry[128]; /* line for log and screen */

    sprintf(entry, "Adding user record %ld", usernum);
    printf("+ %s\n", entry);
    writelog(LOGMINOR, entry);
}

/* scrlog_useredit() - editing user ... */
void scrlog_useredit(long usernum)
{
    char entry[128]; /* line for log and screen */

    sprintf(entry, "Editing user record %ld", usernum);
    printf("+ %s\n", entry);
    writelog(LOGMINOR, entry);
}

/* scrlog_userdelete() - editing user ... */
void scrlog_userdelete(long usernum)
{
    char entry[128]; /* line for log and screen */

    sprintf(entry, "Deleting user record %ld", usernum);
    printf("+ %s\n", entry);
    writelog(LOGMINOR, entry);
}

/* scrlog_userlist() - listing users */
void scrlog_userlist(void)
{
    char entry[128]; /* line for log and screen */

    strcpy(entry, "Listing users");
    printf("+ %s\n", entry);
    writelog(LOGMINOR, entry);
}


/* player database related messages ***********************************/


/* scrlog_plistheader() - display player list header */
void scrlog_plistheader(void)
{
  char entry[128]; /* line for log and screen */

  /* "this is a list" log message */
  strcpy(entry, "Listing players");
  printf("+ %s\n", entry);
  writelog(LOGMINOR, entry);

  /* list header on the screen */
  puts("----------empire----usernum-");
}

/* scrlog_plistbody() - display an entry in the player list body */
void scrlog_plistbody(char *name, long unum)
{
  printf("%16s %10ld\n", name, unum);
}

/* scrlog_plistfooter() - display the list footer */
void scrlog_plistfooter(void)
{
  puts("----------------------------");
}

/* scrlog_playeradd() - adding player ... */
void scrlog_playeradd(char *ename)
{
    char entry[128]; /* line for log and screen */

    sprintf(entry, "Adding player record %s", ename);
    printf("+ %s\n", entry);
    writelog(LOGMINOR, entry);
}

/* scrlog_playeredit() - editing player ... */
void scrlog_playeredit(char *ename)
{
    char entry[128]; /* line for log and screen */

    sprintf(entry, "Editing player record %s", ename);
    printf("+ %s\n", entry);
    writelog(LOGMINOR, entry);
}

/* scrlog_playerdelete() - editing player ... */
void scrlog_playerdelete(char *ename)
{
    char entry[128]; /* line for log and screen */

    sprintf(entry, "Deleting player record %s", ename);
    printf("+ %s\n", entry);
    writelog(LOGMINOR, entry);
}

/* scrlog_playerlist() - listing players */
void scrlog_playerlist(void)
{
    char entry[128]; /* line for log and screen */

    strcpy(entry, "Listing players");
    printf("+ %s\n", entry);
    writelog(LOGMINOR, entry);
}

/* debug output */
void scrlog_debug (char *entry) {
  writelog (LOGDEBUG, entry);
}
