/*
 * user.c
 * Copyright (C) Damian Walker 2000
 *
 * Astroserv - User Database File Maintenance
 *
 * $Id: user.c,v 1.2 2000/07/27 09:31:48 damian Exp $
 *
 * $Log: user.c,v $
 * Revision 1.2  2000/07/27 09:31:48  damian
 * Cosmetic changes to heading comments
 *
 * Revision 1.1.1.1  2000/07/27 09:18:14  damian
 * Initial release of Astroserv
 *
 * Revision 1.1  2000/05/31 10:17:52  damian
 * Initial revision
 *
 */


/* included headers *******************************************/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "astrosrv.h"


/* some macros ************************************************/


/* record sizes */
#define DATASIZE 77  /* size of packed data record */
#define IND1SIZE  8  /* size of packed number index record */
#define IND2SIZE 76  /* size of packed name index record */

/* flags */
#define UF_DELETED 0x01 /* deleted flag */


/* enumerated types *******************************************/


/* user index */
typedef enum {
    U_NONE,   /* read without index */
    U_NUMBER, /* read by number index */
    U_ADDR,   /* read by name index */
    U_LAST    /* placeholder */
} uindex;


/* structures *************************************************/


/* hidden user structure */
typedef struct {

    /* record fields */
    char  deleted;   /* deleted and other flags */
    long  number;    /* user number */
    char  addr[72];  /* user email address */

    /* index maintenance fields */
    char  infile;    /* 0=new record, !0=somewhere in file */
    long  pos;       /* record position when last read */
    long  onumber;   /* original user number */
    char  oaddr[72]; /* original user name */

} user;

/* indexes */
typedef struct {   /* user number index */
    long pos,          /* position of record in data file */
         number;       /* user number */
} ind1;
typedef struct {   /* user addr index */
    long pos;          /* position of record in data file */
    char addr[72];     /* user addr */
} ind2;


/* global variables *******************************************/


FILE *user_data, /* user data file */
     *user_ind1, /* user number index file */
     *user_ind2; /* user addr index file */


/* level 3 routines *******************************************/


/* user_readind1() - read a number index record */
result user_readind1(ind1 *i)
{
    long pos; /* first byte of position in file */

    if(( pos = fgetc(user_ind1) ) == EOF)
        return R_EOF;
    i->pos = pos;
    FREADLONGR(i->pos, user_ind1);
    FREADLONG(i->number, user_ind1);
    return R_OK;
}

/* user_readind2() - read a addr index record */
result user_readind2(ind2 *j)
{
    long pos; /* first byte of position in file */

    if(( pos = fgetc(user_ind2) ) == EOF)
        return R_EOF;
    j->pos = pos;
    FREADLONGR(j->pos, user_ind2);
    fread(j->addr, 72, 1, user_ind2);
    return R_OK;
}


/* level 2 routines *******************************************/


/* user_findind1() - find a number index record */
long user_findind1(ind1 *i, long number)
{
    long pos,    /* original position in index file */
         start,  /* lower bound of binary search area */
         middle, /* midpoint of binary search area */
         end;    /* upper bound of binary search area */

    /* store current file pointer in case of no find */
    pos = ftell(user_ind1); fseek(user_ind1, 0, SEEK_END);
    if( ftell(user_ind1) == 8 ) return 0;

    /* initialise start/end/middle of binary search */
    end = ( ftell(user_ind1) - 8 ) / IND1SIZE - 1;
    start = 0; middle = start + (end - start) / 2;

    /* proceed with binary search */
    fseek(user_ind1, 8 + IND1SIZE * middle, SEEK_SET);
    user_readind1(i);
    while(i->number != number && start <= end)
    {
        if(i->number < number)
            start = middle + 1;
        else
            end = middle - 1;
        middle = start + (end - start) / 2;
        fseek(user_ind1, 8 + IND1SIZE * middle, SEEK_SET);
        user_readind1(i);
    }

    /* return value */
    if(i->number == number)
        return 8 + middle * IND1SIZE;
    fseek(user_ind1, pos, SEEK_SET);
    return 0;
}

/* user_findind2() - find a addr index record */
long user_findind2(ind2 *j, char *addr)
{
    long pos,    /* original position in index file */
         start,  /* lower bound of binary search area */
         middle, /* midpoint of binary search area */
         end;    /* upper bound of binary search area */

    /* store current file pointer in case of no find */
    pos = ftell(user_ind2); fseek(user_ind2, 0, SEEK_END);
    if( ftell(user_ind2) == 8 ) return 0;

    /* initialise start/end/middle of binary search */
    end = ( ftell(user_ind2) - 8 ) / IND2SIZE - 1;
    start = 0; middle = start + (end - start) / 2;

    /* proceed with binary search */
    fseek(user_ind2, 8 + IND2SIZE * middle, SEEK_SET);
    user_readind2(j);
    while( strcasecmp(j->addr, addr) && start <= end)
    {
        if( strcasecmp(j->addr, addr) < 0 )
            start = middle + 1;
        else
            end = middle - 1;
        middle = start + (end - start) / 2;
        fseek(user_ind2, 8 + IND2SIZE * middle, SEEK_SET);
        user_readind2(j);
    }

    if( !strcasecmp(j->addr, addr) )
        return 8 + middle * IND2SIZE;
    fseek(user_ind2, pos, SEEK_SET);
    return 0;
}

/* user_writeind1() - write a number index record */
result user_writeind1(ind1 i)
{
    if( fputc(i.pos & 0xff, user_ind1) == EOF ) return R_FILE;
    FWRITELONGR(i.pos,   user_ind1);
    FWRITELONG(i.number, user_ind1);
    return R_OK;
}

/* user_writeind2() - write a addr index record */
result user_writeind2(ind2 j)
{
    if( fputc(j.pos & 0xff, user_ind2) == EOF ) return R_FILE;
    FWRITELONGR(j.pos, user_ind2);
    fwrite(j.addr, 72, 1, user_ind2);
    return R_OK;
}


/* level 1 routines *******************************************/


/* user_readdata() - read a data record */
result user_readdata(user *u)
{
    int  deleted; /* deleted flag */
    long pos;     /* position in file */

    /* check for EOF */
    pos = ftell(user_data);
    if(( deleted = fgetc(user_data) ) == EOF)
        return R_EOF;

    /* read rest of record */
    u->deleted = deleted;
    FREADLONG(u->number,      user_data);
    fread(u->addr, 72, 1,     user_data);

    /* set index maintenance fields and return */
    u->infile = 1;
    u->pos = pos;
    u->onumber = u->number;
    strcpy(u->oaddr, u->addr);
    return R_OK;
}

/* user_writedata() - write a data record */
result user_writedata(user *u)
{
    long pos; /* position in file */

    /* check that record can be written */
    pos = ftell(user_data);
    if( fputc(u->deleted, user_data) == EOF )
        return R_FILE;

    /* write rest of record */
    FWRITELONG(u->number,      user_data);
    fwrite(u->addr, 72, 1,     user_data);

    /* set index maintenance fields and return */
    u->infile = 1;
    u->pos = pos;
    u->onumber = u->number;
    strcpy(u->oaddr, u->addr);
    return R_OK;
}

/* user_sortind1() - single bi-directional pass of sort */
int user_sortind1(void)
{
    long   pos,    /* stored position in index */
           number; /* original user number of index record */
    ind1   i1,     /* first index record to compare */
           i2;     /* second index record to compare */
    result r;      /* returned to calling process */

    /* obtain information about current position */
    pos = ftell(user_ind1);
    if(pos > 8)
    {
        fseek(user_ind1, -IND1SIZE, SEEK_CUR);
        user_readind1(&i1);
        number = i1.number;
    }
    else number = 0;

    /* ensure file is big enough to need sorting */
    fseek(user_ind1, 0, SEEK_END);
    if( ftell(user_ind1) <= 8 + IND1SIZE )
    {
        fseek(user_ind1, pos, SEEK_SET);
        return R_OK;
    }
    r = R_OK;

    /* forward pass of sort */
    fseek(user_ind1, 8, SEEK_SET); user_readind1(&i1);
    while( user_readind1(&i2) == R_OK )
    {
        if(i1.number > i2.number)
        {
            fseek(user_ind1, -2 * IND1SIZE, SEEK_CUR);
            user_writeind1(i2); user_writeind1(i1);
            r = R_SWAPS;
        }
        fseek(user_ind1, -IND1SIZE, SEEK_CUR);
        user_readind1(&i1);
    }

    /* backward pass of sort */
    fseek(user_ind1, -IND1SIZE, SEEK_END);
    while( ftell(user_ind1) > 8 )
    {
        fseek(user_ind1, -IND1SIZE, SEEK_CUR);
        user_readind1(&i1); user_readind1(&i2);
        if(i1.number > i2.number)
        {
            fseek(user_ind1, -2 * IND1SIZE, SEEK_CUR);
            user_writeind1(i2); user_writeind1(i1);
            r = R_SWAPS;
        }
        fseek(user_ind1, -2 * IND1SIZE, SEEK_CUR);
    }

    /* clean up */
    if(pos > 8)
        user_findind1(&i1, number);
    else
        fseek(user_ind1, pos, SEEK_SET);
    return r;
}

/* user_sortind2() - single bi-directional pass of sort */
int user_sortind2(void)
{
    long   pos;      /* stored position in index */
    ind2   j1,       /* first index record to compare */
           j2;       /* second index record to compare */
    char   addr[72]; /* addr field of current index record */
    result r;        /* returned to calling process */

    /* obtain information about current position */
    pos = ftell(user_ind2);
    if(pos > 8)
    {
        fseek(user_ind2, -IND2SIZE, SEEK_CUR);
        user_readind2(&j1);
        strcpy(addr, j1.addr);
    }

    /* ensure file is big enough to need sorting */
    fseek(user_ind2, 0, SEEK_END);
    if( ftell(user_ind2) <= 8 + IND2SIZE )
    {
        fseek(user_ind2, pos, SEEK_SET);
        return R_OK;
    }
    r = R_OK;

    /* forward pass of sort */
    fseek(user_ind2, 8, SEEK_SET); user_readind2(&j1);
    while( user_readind2(&j2) == R_OK )
    {
        if( strcasecmp(j1.addr, j2.addr) > 0 )
        {
            fseek(user_ind2, -2 * IND2SIZE, SEEK_CUR);
            user_writeind2(j2); user_writeind2(j1);
            r = R_SWAPS;
        }
        fseek(user_ind2, -IND2SIZE, SEEK_CUR);
        user_readind2(&j1);
    }

    /* backward pass of sort */
    fseek(user_ind2, -IND2SIZE, SEEK_END);
    while( ftell(user_ind2) > 8 )
    {
        fseek(user_ind2, -IND2SIZE, SEEK_CUR);
        user_readind2(&j1); user_readind2(&j2);
        if( strcasecmp(j1.addr, j2.addr) > 0 )
        {
            fseek(user_ind2, -2 * IND2SIZE, SEEK_CUR);
            user_writeind2(j2); user_writeind2(j1);
            r = R_SWAPS;
        }
        fseek(user_ind2, -2 * IND2SIZE, SEEK_CUR);
    }

    /* clean up */
    if(pos > 8)
        user_findind2(&j1, addr);
    else
        fseek(user_ind2, pos, SEEK_SET);
    return r;
}


/* level 0 routines *******************************************/


/* user_open() - open user files */
result user_open(char *path)
{
    char dataaddr[128], /* addr of data file */
         ind1addr[128], /* addr of user number index file */
         ind2addr[128], /* addr of user addr index file */
         dataheader[8], /* data file header */
         ind1header[8], /* index 1 header */
         ind2header[8]; /* index 2 header */

    /* initialise data addrs */
    sprintf(dataaddr, "%ssuser.data", path);
    sprintf(ind1addr, "%ssuser1.index", path);
    sprintf(ind2addr, "%ssuser2.index", path);

    /* attempt to open files */
    if(( user_data = fopen(dataaddr, "r+b") ) == NULL)
        return R_FILE;
    if(( user_ind1 = fopen(ind1addr, "r+b") ) == NULL)
    {
        fclose(user_data);
        return R_FILE;
    }
    if(( user_ind2 = fopen(ind2addr, "r+b") ) == NULL)
    {
        fclose(user_data);
        fclose(user_ind1);
        return R_FILE;
    }

    /* verify files */
    fread(dataheader, 8, 1, user_data);
    fread(ind1header, 8, 1, user_ind1);
    fread(ind2header, 8, 1, user_ind2);
    if(( strncmp(dataheader, "ASS10UF", 8) ) ||
       ( strncmp(ind1header, "ASS10U1", 8) ) ||
       ( strncmp(ind2header, "ASS10U2", 8) ))
    {
        fclose(user_data);
        fclose(user_ind1);
        fclose(user_ind2);
        return R_HEADER;
    }

    /* files checked out OK */
    return R_OK;
}

/* user_create() - create user files */
result user_create(char *path)
{
    char  dataaddr[128], /* addr of data file */
          ind1addr[128], /* addr of user number index file */
          ind2addr[128]; /* addr of user addr index file */
    FILE *test;          /* used to test existence of files */

    /* initialise data addrs */
    sprintf(dataaddr, "%ssuser.data", path);
    sprintf(ind1addr, "%ssuser1.index", path);
    sprintf(ind2addr, "%ssuser2.index", path);

    /* check for files' existence */
    if(( test = fopen(dataaddr, "rb") ) != NULL)
    {
        fclose(test);
        return R_EXISTS;
    }
    if(( test = fopen(ind1addr, "rb") ) != NULL)
    {
        fclose(test);
        return R_EXISTS;
    }
    if(( test = fopen(ind1addr, "rb") ) != NULL)
    {
        fclose(test);
        return R_EXISTS;
    }

    /* attempt to create files */
    if(( user_data = fopen(dataaddr, "wb") ) == NULL)
        return R_FILE;
    if(( user_ind1 = fopen(ind1addr, "wb") ) == NULL)
    {
        fclose(user_data); remove(dataaddr);
        return R_FILE;
    }
    if(( user_ind2 = fopen(ind2addr, "wb") ) == NULL)
    {
        fclose(user_data); remove(dataaddr);
        fclose(user_ind1); remove(ind1addr);
        return R_FILE;
    }

    /* write headers */
    fwrite("ASS10UF", 8, 1, user_data);
    fwrite("ASS10U1", 8, 1, user_ind1);
    fwrite("ASS10U2", 8, 1, user_ind2);

    /* close files and return */
    fclose(user_data);
    fclose(user_ind1);
    fclose(user_ind2);
    return R_OK;
}

/* user_close() - close user files */
result user_close(void)
{
    fclose(user_data);
    fclose(user_ind1);
    fclose(user_ind2);
    return R_OK;
}

/* user_new() - allocate memory for new user record */
user *user_new(void)
{
    return calloc( 1, sizeof(user) );
}

/* user_old() - free memory for new user record */
void user_old(user *u)
{
    free(u);
}

/* user_clear() - clear details from existing user record */
void user_clear(user *u)
{
    memset( u, 0, sizeof(user) );
}

/* user_setnumber() - set the user number */
void user_setnumber(user *u, long number)
{
    u->number = number;
}

/* user_getnumber() - return the user number */
long user_getnumber(user *u)
{
    return u->number;
}

/* user_setaddr() - set the user addr */
void user_setaddr(user *u, char *addr)
{
    strcpy(u->addr, addr);
}

/* user_getaddr() - return the user addr */
char *user_getaddr(char *addr, user *u)
{
    return strcpy(addr, u->addr);
}

/* user_write() - write a record */
result user_write(user *u)
{
    ind1   i;  /* number index record */
    ind2   j;  /* addr index record */
    long   p1, /* position of index record 1 */
           p2; /* position of index record 2 */

    /* write new record */
    if(!u->infile)
    {
        if( user_findind1(&i, u->number) ) return R_DUPE;
        if( user_findind2(&j, u->addr) )   return R_DUPE;
        fseek(user_data, 0, SEEK_END);
        fseek(user_ind1, 0, SEEK_END);
        fseek(user_ind2, 0, SEEK_END);
        i.pos = j.pos = ftell(user_data);
        i.number = u->number;
        strcpy(j.addr, u->addr);
        if( user_writedata(u) != R_OK )    return R_FILE;
        if( user_writeind1(i) != R_OK )    return R_CORRUPT;
        user_sortind1();
        if( user_writeind2(j) != R_OK )    return R_CORRUPT;
        user_sortind2();
        return R_OK;
    }

    /* rewrite existing record */
    p1 = user_findind1(&i, u->onumber);
    p2 = user_findind2(&j, u->oaddr);
    if( (!p1 || !p2) )                     return R_CORRUPT;
    if( (i.pos != j.pos) )                 return R_CORRUPT;
    fseek(user_data, i.pos, SEEK_SET);
    if( user_writedata(u) != R_OK )        return R_FILE;
    if(i.number != u->number)
    {
        i.number = u->number;
        fseek(user_ind1, p1, SEEK_SET);
        if( user_writeind1(i) != R_OK )    return R_CORRUPT;
        user_sortind1();
    }
    if( strcasecmp(j.addr, u->addr) )
    {
        strcpy(j.addr, u->addr);
        fseek(user_ind2, p2, SEEK_SET);
        if( user_writeind2(j) != R_OK )    return R_CORRUPT;
        user_sortind2();
    }
    return R_OK;
}

/* user_first() - read the first record */
result user_first(user *u, uindex inum)
{
    ind1   i; /* number index record */
    ind2   j; /* addr index record */
    result r;  /* result of index read */

    switch(inum)
    {
        case U_NONE:
            fseek(user_data, 8, SEEK_SET);
            return user_readdata(u);
        case U_NUMBER:
            fseek(user_ind1, 8, SEEK_SET);
            if(( r = user_readind1(&i) ) != R_OK) return r;
            fseek(user_data, i.pos, SEEK_SET);
            return user_readdata(u);
        case U_ADDR:
            fseek(user_ind2, 8, SEEK_SET);
            if(( r = user_readind2(&j) ) != R_OK) return r;
            fseek(user_data, j.pos, SEEK_SET);
            return user_readdata(u);
        default:
            return R_INDEX;
    }
}

/* user_next() - read the next record */
result user_next(user *u, uindex inum)
{
    ind1   i; /* number index record */
    ind2   j; /* addr index record */
    result r;  /* result of index read */

    switch(inum)
    {
        case U_NONE:
            return user_readdata(u);
        case U_NUMBER:
            if(( r = user_readind1(&i) ) != R_OK) return r;
            fseek(user_data, i.pos, SEEK_SET);
            return user_readdata(u);
        case U_ADDR:
            if(( r = user_readind2(&j) ) != R_OK) return r;
            fseek(user_data, j.pos, SEEK_SET);
            return user_readdata(u);
        default:
            return R_INDEX;
    }
}

/* user_prev() - read the previous record */
result user_prev(user *u, uindex inum)
{
    ind1   i; /* number index record */
    ind2   j; /* addr index record */
    result r;  /* result of index read */

    switch(inum)
    {
        case U_NONE:
            if(( ftell(user_data) - 2 * DATASIZE ) < 8)
                return R_EOF;
            fseek(user_data, -2 * DATASIZE, SEEK_CUR);
            return user_readdata(u);
        case U_NUMBER:
            if(( ftell(user_ind1) - 2 * IND1SIZE ) < 8)
                return R_EOF;
            fseek(user_ind1, -2 * IND1SIZE, SEEK_CUR);
            if(( r = user_readind1(&i) ) != R_OK) return r;
            fseek(user_data, i.pos, SEEK_SET);
            return user_readdata(u);
        case U_ADDR:
            if(( ftell(user_ind2) - 2 * IND2SIZE ) < 8)
                return R_EOF;
            fseek(user_ind2, -2 * IND2SIZE, SEEK_CUR);
            if(( r = user_readind2(&j) ) != R_OK) return r;
            fseek(user_data, j.pos, SEEK_SET);
            return user_readdata(u);
        default:
            return R_INDEX;
    }
}

/* user_last() - read the last record */
result user_last(user *u, uindex inum)
{
    ind1   i;   /* number index record */
    ind2   j;   /* addr index record */
    result r;   /* result of index read */
    long   pos; /* current data/index file position */

    switch(inum)
    {
        case U_NONE:
            pos = ftell(user_data);
            if( fseek(user_data, -DATASIZE, SEEK_END) )
                return R_EOF;
            if( ftell(user_data) < 8 )
            {
                fseek(user_data, pos, SEEK_SET);
                return R_EOF;
            }
            return user_readdata(u);
        case U_NUMBER:
            pos = ftell(user_ind1);
            if( fseek(user_ind1, -IND1SIZE, SEEK_END) )
                return R_EOF;
            if( ftell(user_ind1) < 8 )
            {
                fseek(user_ind1, pos, SEEK_SET);
                return R_EOF;
            }
            if(( r = user_readind1(&i) ) != R_OK) return r;
            fseek(user_data, i.pos, SEEK_SET);
            return user_readdata(u);
        case U_ADDR:
            pos = ftell(user_ind2);
            if( fseek(user_ind2, -IND2SIZE, SEEK_END) )
                return R_EOF;
            if( ftell(user_ind2) < 8 )
            {
                fseek(user_ind2, pos, SEEK_SET);
                return R_EOF;
            }
            if(( r = user_readind2(&j) ) != R_OK) return r;
            fseek(user_data, j.pos, SEEK_SET);
            return user_readdata(u);
        default:
            return R_INDEX;
    }
}

/* user_find() - find a record by number or addr */
result user_find(user *u, char *searchtext)
{
    ind1   i;      /* user number index record */
    ind2   j;      /* user addr index record */
    long   number; /* user number from search text */

    if( sscanf(searchtext, "%ld", &number) == 1 )
    {
        if( user_findind1(&i, number) == 0 ) return R_EOF;
        fseek(user_data, i.pos, SEEK_SET);
        return user_readdata(u);
    }
    if( user_findind2(&j, searchtext) == 0 ) return R_EOF;
    fseek(user_data, j.pos, SEEK_SET);
    return user_readdata(u);

    return R_OK;
}

/* user_delete() - mark a record as deleted */
result user_delete(user *u)
{
    if(u->pos == 0) return R_EOF;
    u->deleted ^= UF_DELETED;
    fseek(user_data, u->pos, SEEK_SET);
    return user_writedata(u);
}

/* user_deleted() - return the deleted status of a record */
int user_deleted(user *u)
{
    if(u->pos == 0) return R_EOF;
    return u->deleted & UF_DELETED;
}

/* user_pack() - pack a close data file */
result user_pack(char *path)
{
    user  *u;             /* user record */
    char   dataaddr[128], /* addr of data file */
           ind1addr[128], /* addr of user number index file */
           ind2addr[128], /* addr of user addr index file */
           tempaddr[128], /* addr of temporary file */
           dataheader[8]; /* data file header */
    FILE  *user_temp;     /* temporary file */
    ind1   i;             /* number index record */
    ind2   j;             /* addr index record */

    /* initialise data addrs */
    sprintf(dataaddr, "%ssuser.data", path);
    sprintf(tempaddr, "%ssuser.temp", path);

    /* attempt to open data & temp file */
    if(( user_data = fopen(dataaddr, "r+b") ) == NULL)
        return R_FILE;
    fread(dataheader, 8, 1, user_data);
    if(( strncmp(dataheader, "ASS10UF", 8) ) ||
       ( user_temp = fopen(tempaddr, "w+b") ) == NULL)
    {
        fclose(user_data);
        return R_HEADER;
    }

    /* allocate memory */
    if( (u = malloc( sizeof(user) )) == NULL )
    {
        fclose(user_data);
        fclose(user_temp);
        remove(tempaddr);
        return R_MEMORY;
    }

    /* copy non-deleted records to temporary file and back */
    while( user_readdata(u) == R_OK )
        if( !(u->deleted & UF_DELETED) )
            fwrite(u, sizeof(user), 1, user_temp);
    fclose(user_data);
    remove(dataaddr);
    if(( user_data = fopen(dataaddr, "w+b") ) == NULL)
    {
        free(u);
        fclose(user_temp);
        remove(tempaddr);
        return R_FILE;
    }
    fwrite("ASS10UF", 8, 1, user_data);
    fseek(user_temp, 0, SEEK_SET);
    while( fread(u, sizeof(user), 1, user_temp) == 1 )
        user_writedata(u);
    fclose(user_temp);
    remove(tempaddr);

    /* recreate number index */
    sprintf(ind1addr, "%ssuser1.index", path); remove(ind1addr);
    if(( user_ind1 = fopen(ind1addr, "w+b") ) == NULL)
    {
        free(u);
        fclose(user_data);
        return R_FILE;
    }
    fwrite("ASS10U1", 8, 1, user_ind1);
    fseek(user_data, 8, SEEK_SET); i.pos = ftell(user_data);
    while( user_readdata(u) == R_OK )
    {
        i.number = u->number;
        user_writeind1(i);
        i.pos = ftell(user_data);
    }
    while( user_sortind1() == R_SWAPS );
    fclose(user_ind1);

    /* recreate addr index */
    sprintf(ind2addr, "%ssuser2.index", path); remove(ind2addr);
    if(( user_ind2 = fopen(ind2addr, "w+b") ) == NULL)
    {
        free(u);
        fclose(user_data);
        return R_FILE;
    }
    fwrite("ASS10U2", 8, 1, user_ind2);
    fseek(user_data, 8, SEEK_SET); j.pos = ftell(user_data);
    while( user_readdata(u) == R_OK )
    {
        strcpy(j.addr, u->addr);
        user_writeind2(j);
        j.pos = ftell(user_data);
    }
    while( user_sortind2() == R_SWAPS );
    fclose(user_ind2);

    /* clean up */
    free(u);
    fclose(user_data);
    return R_OK;
}
